@@ -22,6 +22,7 @@
 #include "exm-browse-page.h"
 #include "exm-installed-page.h"
 #include "exm-detail-view.h"
+#include "exm-release-notes-dialog.h"
 
 #include "local/exm-manager.h"
 #include "local/exm-extension.h"
@@ -198,6 +199,31 @@ on_install_done (GObject       *source,
     }
 }
 
+typedef struct
+{
+    ExmManager *manager;
+    gchar *uuid;
+} UnsupportedDialogData;
+
+static void
+extension_unsupported_dialog_response (GtkDialog             *dialog,
+                                       int                    response_id,
+                                       UnsupportedDialogData *data)
+{
+    gtk_window_destroy (GTK_WINDOW (dialog));
+
+    if (response_id == GTK_RESPONSE_YES)
+    {
+        exm_manager_install_async (data->manager, data->uuid, NULL,
+                                   (GAsyncReadyCallback) on_install_done,
+                                   NULL);
+    }
+
+    g_clear_pointer (&data->manager, g_object_unref);
+    g_clear_pointer (&data->uuid, g_free);
+    g_free (data);
+}
+
 static void
 extension_install (GtkWidget  *widget,
                    const char *action_name,
@@ -205,9 +231,30 @@ extension_install (GtkWidget  *widget,
 {
     ExmWindow *self;
     gchar *uuid;
+    gboolean warn;
 
     self = EXM_WINDOW (widget);
-    g_variant_get (param, "s", &uuid);
+    g_variant_get (param, "(sb)", &uuid, &warn);
+
+    if (warn)
+    {
+        GtkWidget *dlg;
+
+        dlg = gtk_message_dialog_new (GTK_WINDOW (self),
+                                      GTK_DIALOG_MODAL,
+                                      GTK_MESSAGE_QUESTION,
+                                      GTK_BUTTONS_YES_NO,
+                                      _("This extension does not support your GNOME Shell version.\nWould you like to install anyway?"));
+
+        UnsupportedDialogData *data = g_new0 (UnsupportedDialogData, 1);
+        data->manager = g_object_ref (self->manager);
+        data->uuid = g_strdup (uuid);
+
+        g_signal_connect (dlg, "response", G_CALLBACK (extension_unsupported_dialog_response), data);
+        gtk_widget_show (dlg);
+
+        return;
+    }
 
     exm_manager_install_async (self->manager, uuid, NULL,
                                (GAsyncReadyCallback) on_install_done,
@@ -226,12 +273,11 @@ show_view (GtkWidget  *widget,
     if (g_str_equal (action_name, "win.show-detail"))
     {
         gchar *uuid;
-        int pk;
 
-        g_variant_get (param, "s", &uuid, &pk);
-        adw_leaflet_set_visible_child (self->leaflet, self->detail_view);
+        g_variant_get (param, "s", &uuid);
+        adw_leaflet_set_visible_child (self->leaflet, GTK_WIDGET (self->detail_view));
 
-        exm_detail_view_load_for_uuid (self->detail_view, uuid, pk);
+        exm_detail_view_load_for_uuid (self->detail_view, uuid);
 
         return;
     }
@@ -239,6 +285,21 @@ show_view (GtkWidget  *widget,
     adw_leaflet_set_visible_child (self->leaflet, self->main_view);
 }
 
+static void
+show_release_notes (GtkWidget  *widget,
+                    const char *action_name,
+                    GVariant   *param)
+{
+    ExmWindow *self;
+
+    self = EXM_WINDOW (widget);
+
+    ExmReleaseNotesDialog *notes = exm_release_notes_dialog_new ();
+    gtk_window_set_modal (GTK_WINDOW (notes), TRUE);
+    gtk_window_set_transient_for (GTK_WINDOW (notes), GTK_WINDOW (self));
+    gtk_window_present (GTK_WINDOW (notes));
+}
+
 static void
 exm_window_class_init (ExmWindowClass *klass)
 {
@@ -271,12 +332,29 @@ exm_window_class_init (ExmWindowClass *klass)
     // TODO: Refactor ExmWindow into a separate ExmController and supply the
     // necessary actions/methods/etc in there. A reference to this new object can
     // then be passed to each page.
-    gtk_widget_class_install_action (widget_class, "ext.install", "s", extension_install);
+    gtk_widget_class_install_action (widget_class, "ext.install", "(sb)", extension_install);
     gtk_widget_class_install_action (widget_class, "ext.remove", "s", extension_remove);
     gtk_widget_class_install_action (widget_class, "ext.state-set", "(sb)", extension_state_set);
     gtk_widget_class_install_action (widget_class, "ext.open-prefs", "s", extension_open_prefs);
     gtk_widget_class_install_action (widget_class, "win.show-detail", "s", show_view);
     gtk_widget_class_install_action (widget_class, "win.show-main", NULL, show_view);
+    gtk_widget_class_install_action (widget_class, "win.show-release-notes", NULL, show_release_notes);
+}
+
+static void
+do_version_check (ExmWindow *self)
+{
+    GSettings *settings;
+    gchar *version_string;
+
+    settings = g_settings_new (APP_ID);
+    version_string = g_settings_get_string (settings, "last-used-version");
+
+    // In the future, use this to show a toast or notification when
+    // a new version has been installed. Maybe with rich text release notes?
+    // if (strcmp (version_string, APP_VERSION) != 0) { ... }
+
+    g_settings_set_string (settings, "last-used-version", APP_VERSION);
 }
 
 static void
@@ -284,8 +362,15 @@ exm_window_init (ExmWindow *self)
 {
     gtk_widget_init_template (GTK_WIDGET (self));
 
+    if (IS_DEVEL) {
+        gtk_widget_add_css_class (GTK_WIDGET (self), "devel");
+    }
+
     self->manager = exm_manager_new ();
 
+    gtk_window_set_title (GTK_WINDOW (self),
+                          IS_DEVEL ? _("Extension Manager (Development)") : _("Extension Manager"));
+
     g_object_set (self->installed_page, "manager", self->manager, NULL);
     g_object_set (self->browse_page, "manager", self->manager, NULL);
     g_object_set (self->detail_view, "manager", self->manager, NULL);
@@ -301,4 +386,7 @@ exm_window_init (ExmWindow *self)
                             self->global_toggle,
                             "state",
                             G_BINDING_BIDIRECTIONAL|G_BINDING_SYNC_CREATE);
+
+    // Window must be mapped to show version check dialog
+    g_signal_connect (self, "map", G_CALLBACK (do_version_check), NULL);
 }
